/* For license details see bottom.
 * Copyright (c) 2002 Catalyst of Design (David Morris-Oliveros).  All rights reserved.
 */

// system includes
#include <caosGL/core/globals.h>
#include <caosGL/core/types.h>

// package includes
#include <caosGL/gfx/cScene.h>

// extern includes
#include <caosGL/core/cParser.h>
#include <caosGL/core/Dyngl.h>

#include <caosGL/gfx/cTransform.h>

namespace caosGL {
	namespace gfx {
		/**
		 *<br> class:		cTransform
		 *<br> namespace:	caosGL::gfx
		 *<br> inherits:	caosGL::gfx::cGroup
		 *<br> implements:	<none>
		 *<br> purpose:		Groups together objects under a geometric transformation
		 *
		 */

		static vector <tAttr> attributeNames;
		static log4cpp::Category& cat = log4cpp::Category::getInstance ("caosGL::gfx::cTransform");

		/********************************************************************************************/
		cTransform::cTransform (const string & n, cGroup * f) : super (n,f) {
			init ();
		}

		/********************************************************************************************/
		cTransform::~cTransform () {
		}

		/********************************************************************************************/
		tVoid cTransform::matrix (const cMatrix & m) {
			_matrix = m;
		}

		/********************************************************************************************/
		const cMatrix & cTransform::matrix () const {
			return _matrix;
		}

		/********************************************************************************************/
		cMatrix & cTransform::matrix () {
			return _matrix;
		}

		/********************************************************************************************/
		tVoid cTransform::visitInverted (tFloat t) {
			evaluateAll (t);
			glScalef (_sX, _sY, _sZ);
			glRotatef (_rX, -1, 0, 0);
			glRotatef (_rY, 0, -1, 0);
			glRotatef (_rZ, 0, 0, -1);
			glTranslatef (-_tX, -_tY, -_tZ);
		}

		/********************************************************************************************/
		tVoid cTransform::leaveInverted () {
		}

		// from cBaseNode
		/********************************************************************************************/
		tBool cTransform::compile () {
			return super::compile ();
		}

		// from cBaseNode
		/********************************************************************************************/
		tBool cTransform::visit (tFloat t) {
			evaluateAll (t);
			glMatrixMode (GL_MODELVIEW);
			glPushMatrix ();
			glTranslatef (_tX, _tY, _tZ);
			glRotatef (_rX, 1, 0, 0);
			glRotatef (_rY, 0, 1, 0);
			glRotatef (_rZ, 0, 0, 1);
			glScalef (_sX, _sY, _sZ);
			return true;
		}

		// from cBaseNode
		/********************************************************************************************/
		tVoid cTransform::leave () {
			glMatrixMode (GL_MODELVIEW);
			glPopMatrix ();
			return;
		}

		// from cBaseNode
		/********************************************************************************************/
		tBool cTransform::init () {
			#define ATTRIB(n,t,v,d) ATTRIB_INIT_VAR(n,v)
			#include cTransform_attribsFile
			#undef ATTRIB
			return true;
		}

		// from cBaseNode
		/********************************************************************************************/
		const tBool cTransform::set (const tDWord key, const string & value) {
			if (super::set (key, value)) return true;
			switch (key) {
				#define ATTRIB(n,t,v,d) ATTRIB_SET_S(n)
				#include cTransform_attribsFile
				#undef ATTRIB
			case '    ': return false;
			default: return false;
			}
			return false;
		}

		// from cBaseNode
		/********************************************************************************************/
		const tBool cTransform::set (const tDWord key, const tFloat & value) {
			if (super::set (key, value)) return true;
			switch (key) {
				#define ATTRIB(n,t,v,d) ATTRIB_SET_N(n)
				#include cTransform_attribsFile
				#undef ATTRIB
			case '    ': return false;
			default: return false;
			}
			return false;
		}

		// from cBaseNode
		/********************************************************************************************/
		const tBool cTransform::get (const tDWord key, string & value) const {
			if (super::get (key, value)) return true;
			switch (key) {
				#define ATTRIB(n,t,v,d) ATTRIB_GET(n)
				#include cTransform_attribsFile
				#undef ATTRIB
			case '    ': return false;
			default: return false;
			}
			return false;
		}

		// from cBaseNode
		/********************************************************************************************/
		const vector <tAttr> * cTransform::getAttributeNames () const {
			if (attributeNames.size () == 0) {
				const vector <tAttr> * v = super::getAttributeNames ();
				attributeNames.insert (attributeNames.begin (), v->begin (), v->end ());
				#define ATTRIB(n,t,v,d) ATTRIB_ATTRIBNAMES(n,d)
				#include cTransform_attribsFile
				#undef ATTRIB
			}
			return &attributeNames;
		}

		// from cBaseNode
		/********************************************************************************************/
		const string cTransform::getTypeName () const { return "caosGL::gfx::cTransform"; }
	}
}

// for node creation
#include <caosGL/core/cRegistry.h>
#include <caosGL/gfx/cNodeCreator.h>

class cTransformNodeCreator : public cNodeCreator {
public:
	cTransformNodeCreator () {
		name ("caosGL::gfx::cTransform");
	}
	cBaseNode * createNode (const string n, cGroup * f) {
		return new caosGL::gfx::cTransform (n,f);
	}
};
caosGL::core::cRegisterNodeCreator <cTransformNodeCreator> cTransformNodeCreatorInstance;

/**
 * The Catalyst of Design Software License, Version 1.0
 *
 * Copyright (c) 2002 Catalyst of Design (David Morris-Oliveros).  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by 
 *        Catalyst of Design (http://talsit.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "caosGL" and "Catalyst of Design" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact caosGL@talsit.org.
 *
 * 5. Products derived from this software may not be called "caosGL",
 *    nor may "caosGL" appear in their name, without prior written
 *    permission of Catalyst of Design.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL CATALYST OF DESIGN OR ITS 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
// eof